/*
 * Copyright (c) 2022 Christian Wittenberg
 *
 * thisipcan.cyou gnome extension is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 *
 * thisipcan.cyou gnome extension is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Gnome Documents; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Author:
 * Christian Wittenberg <gnome@ipcan.cyou>
 *
 */

const {St, Clutter} = imports.gi;
const Mainloop = imports.mainloop;
const Main = imports.ui.main;
const Soup = imports.gi.Soup;
const Lang = imports.lang;
const Util = imports.misc.util;
const GLib = imports.gi.GLib;

const thisExtensionDir = GLib.get_home_dir() + '/.local/share/gnome-shell/extensions/external-ip-extension@ipcan.cyou';
const iconLocation = thisExtensionDir + '/img/ip_red.svg';
const extIpService = 'http://thisipcan.cyou';

let panelButton;
let panelButtonText;

let currentIP=""; // stores previously detected external ip
let disabled=false; // stop processing if extension is disabled
let elapsed=0; // time elapsed before next external ip check
let timeout=60*2; // be friendly, refresh every 2 mins.

// returns raw HTTP response
function httpRequest(url, type='GET') {
	let soupSyncSession = new Soup.SessionSync();
    let message = Soup.Message.new(type, url);

    message.request_headers.set_content_type("application/json", null);
    let responseCode = soupSyncSession.send_message(message);
    let out;
    if(responseCode == 200) {
        try {            
			out = message['response-body'].data
        } catch(error) {
            log(error);
        }
    }
    return out;
}

// gets external IP and updates label in toolbar
// if changed, show Gnome notification
function refreshIP() {
	let ipAddress = httpRequest("http://icanhazip.com");

	if(currentIP != "" && currentIP != ipAddress) {
		//new ip address found.
		log('Note: External IP address has been changed into ' + ipAddress + ", send notification")

        //notify user about the switch
		Util.spawn(['/usr/bin/notify-send', 'External IP Address', 'Has been changed to ' + ipAddress, '--icon=' + iconLocation + '']);
	}
	
	currentIP = ipAddress;

    panelButtonText = new St.Label({
        text : ipAddress,
        y_align: Clutter.ActorAlign.CENTER,
    });
    panelButton.set_child(panelButtonText);		
}

// wait until time elapsed, to be friendly to external ip url
function timer(){
    if(!disabled) {        
        if(elapsed >= timeout) {		
            elapsed = 0;

            refreshIP();
        } else {
            elapsed++;
        }	
        
        Mainloop.timeout_add_seconds(1, function() { timer(); });
    }
}

function init () {        
}

function enable () {
    disabled=false;

    panelButton = new St.Bin({
        style_class : "panel-button",
    });
    let panelButtonText = new St.Label({
        text : "IP: <checking>",
        y_align: Clutter.ActorAlign.CENTER,
    });
    panelButton.set_child(panelButtonText);	

    // After enabling, immediately get ip
	refreshIP();

    // Enable timer
	timer();

    // Add the button to the panel
    Main.panel._rightBox.insert_child_at_index(panelButton, 0);
}

function disable () {
    // Set to true so if the timer hits, stop.
    disabled=true;

    // Remove the added button from panel
    Main.panel._rightBox.remove_child(panelButton);
}